@@ -236,6 +236,24 @@ grl_media_add_external_url (GrlMedia *media, const gchar *url)
                        url);
 }
 
+/**
+ * grl_media_add_keyword:
+ * @media: a #GrlMedia
+ * @keyword: a keyword describing the media
+ *
+ * Adds the keyword describing the @media.
+ *
+ * Since: 0.2.3
+ */
+void
+grl_media_add_keyword (GrlMedia *media,
+                       const gchar *keyword)
+{
+  grl_data_add_string (GRL_DATA (media),
+                       GRL_METADATA_KEY_KEYWORD,
+                       keyword);
+}
+
 /**
  * grl_media_serialize:
  * @media: a #GrlMedia
@@ -277,18 +295,25 @@ grl_media_serialize_extended (GrlMedia *media,
                               GrlMediaSerializeType serial_type,
                               ...)
 {
+  GByteArray *binary_blob;
   GList *key;
   GList *keylist;
   GRegex *type_regex;
   GString *serial;
   GrlKeyID grlkey;
   GrlRegistry *registry;
+  GrlRelatedKeys *relkeys;
   const GValue *value;
   const gchar *id;
   const gchar *source;
   const gchar *type_name;
+  gchar *base64_blob;
+  gchar *iso8601_datetime;
   gchar *protocol;
   gchar *serial_media;
+  gchar separator = '?';
+  guint i;
+  guint num_values;
   va_list va_serial;
 
   g_return_val_if_fail (GRL_IS_MEDIA (media), NULL);
@@ -334,7 +359,6 @@ grl_media_serialize_extended (GrlMedia *media,
 
     /* Include all properties */
     if (serial_type == GRL_MEDIA_SERIALIZE_PARTIAL) {
-      g_string_append_c (serial, '?');
       registry = grl_registry_get_default ();
 
       va_start (va_serial, serial_type);
@@ -346,11 +370,25 @@ grl_media_serialize_extended (GrlMedia *media,
             grlkey == GRL_METADATA_KEY_SOURCE) {
           continue;
         }
-        value = grl_data_get (GRL_DATA (media), grlkey);
-        if (value) {
+        num_values = grl_data_length (GRL_DATA (media), grlkey);
+        for (i = 0; i < num_values; i++) {
+
+          g_string_append_c (serial, separator);
+          if (separator == '?') {
+            separator = '&';
+          }
+
           g_string_append_printf (serial,
                                   "%s=",
                                   GRL_METADATA_KEY_GET_NAME (grlkey));
+
+          relkeys = grl_data_get_related_keys (GRL_DATA (media), grlkey, i);
+          if (!grl_related_keys_has_key (relkeys, grlkey)) {
+            continue;
+          }
+
+          value = grl_related_keys_get (relkeys, grlkey);
+
           if (G_VALUE_HOLDS_STRING (value)) {
             g_string_append_uri_escaped (serial,
                                          g_value_get_string (value),
@@ -360,15 +398,29 @@ grl_media_serialize_extended (GrlMedia *media,
             g_string_append_printf (serial, "%d", g_value_get_int (value));
           } else if (G_VALUE_HOLDS_FLOAT (value)) {
             g_string_append_printf (serial, "%f", g_value_get_float (value));
+          } else if (G_VALUE_HOLDS_BOOLEAN (value)) {
+            g_string_append_printf (serial, "%d", g_value_get_boolean (value));
+          } else if (G_VALUE_TYPE (value) == G_TYPE_BYTE_ARRAY) {
+            binary_blob = g_value_get_boxed (value);
+            base64_blob = g_base64_encode (binary_blob->data, binary_blob->len);
+            g_string_append_uri_escaped (serial,
+                                         base64_blob,
+                                         NULL,
+                                         TRUE);
+            g_free (base64_blob);
+          } else if (G_VALUE_TYPE (value) == G_TYPE_DATE_TIME) {
+            iso8601_datetime = g_date_time_format (g_value_get_boxed (value),
+                                                   "%FT%T");
+            g_string_append_uri_escaped (serial,
+                                         iso8601_datetime,
+                                         NULL,
+                                         TRUE);
+            g_free (iso8601_datetime);
           }
-          g_string_append_c (serial, '&');
         }
       }
 
       va_end (va_serial);
-
-      /* Remove trailing ?/& character */
-      g_string_erase (serial, serial->len - 1, -1);
     }
     serial_media = g_string_free (serial, FALSE);
     break;
@@ -379,6 +431,21 @@ grl_media_serialize_extended (GrlMedia *media,
   return serial_media;
 }
 
+static void
+_insert_and_free_related_list (GrlKeyID key,
+                               GList *relkeys_list,
+                               GrlData *data)
+{
+  GList *p = relkeys_list;
+
+  while (p) {
+    grl_data_add_related_keys (data, (GrlRelatedKeys *) p->data);
+    p = g_list_next (p);
+  }
+
+  g_list_free (relkeys_list);
+}
+
 /**
  * grl_media_unserialize:
  * @serial: a serialized media
@@ -392,20 +459,31 @@ grl_media_serialize_extended (GrlMedia *media,
 GrlMedia *
 grl_media_unserialize (const gchar *serial)
 {
+  GDateTime *datetime;
+  GHashTable *grlkey_related_table;
+  GList *keys;
+  GList *relkeys_list;
   GMatchInfo *match_info;
   GRegex *protocol_regex;
   GRegex *query_regex;
   GRegex *uri_regex;
-  GType type_media;
+  GType type_media, type_grlkey;
   GrlKeyID grlkey;
+  GrlKeyID grlkey_index;
   GrlMedia *media;
   GrlRegistry *registry;
+  GrlRelatedKeys *relkeys;
+  gboolean append;
   gchar *escaped_value;
   gchar *keyname;
   gchar *protocol;
   gchar *query;
   gchar *type_name;
   gchar *value;
+  gpointer p;
+  gsize blob_size;
+  guchar *blob;
+  guint *grlkey_count;
 
   g_return_val_if_fail (serial, NULL);
 
@@ -466,33 +544,81 @@ grl_media_unserialize (const gchar *serial)
   g_match_info_free (match_info);
   if (query) {
     registry = grl_registry_get_default ();
+    keys = grl_registry_get_metadata_keys (registry);
+    /* This is a hack: we do it because we know GrlKeyId are actually integers,
+       and assigned sequentially (0 is for invalid key). This saves us to use a
+       hashtable to store the counter per key */
+    grlkey_count = g_new0 (guint, g_list_length(keys) + 1);
+    g_list_free (keys);
+
+    /* In this hashtable we enqueue all the GrlRelatedKeys, that will be added
+       at the end; we can not add them directly because could be we have a
+       GrlRelatedKeys with no values because one of the properties will come
+       later; and we can not add empty values in data */
+    grlkey_related_table = g_hash_table_new (g_direct_hash, g_direct_equal);
+
     query_regex = g_regex_new ("([^=&]+)=([^=&]*)", 0, 0, NULL);
     g_regex_match (query_regex, query, 0, &match_info);
     while (g_match_info_matches (match_info)) {
       keyname = g_match_info_fetch (match_info, 1);
       grlkey = grl_registry_lookup_metadata_key (registry, keyname);
       if (grlkey) {
-        escaped_value = g_match_info_fetch (match_info, 2);
-        value = g_uri_unescape_string (escaped_value, NULL);
-        switch (GRL_METADATA_KEY_GET_TYPE (grlkey)) {
-        case G_TYPE_STRING:
-          grl_data_set_string (GRL_DATA (media), grlkey, value);
-          break;
-        case G_TYPE_INT:
-          grl_data_set_int (GRL_DATA (media), grlkey, atoi (value));
-          break;
-        case G_TYPE_FLOAT:
-          grl_data_set_float (GRL_DATA (media), grlkey, atof (value));
-          break;
+        /* Search for the GrlRelatedKeys to insert the key, or create a new one */
+        grlkey_index =
+          GRLPOINTER_TO_KEYID (g_list_nth_data ((GList *) grl_registry_lookup_metadata_key_relation (registry, grlkey), 0));
+        relkeys_list = g_hash_table_lookup (grlkey_related_table,
+                                            GRLKEYID_TO_POINTER (grlkey_index));
+        p = g_list_nth_data (relkeys_list, grlkey_count[grlkey]);
+        if (p) {
+          relkeys = (GrlRelatedKeys *) p;
+          append = FALSE;
+        } else {
+          relkeys = grl_related_keys_new ();
+          append = TRUE;
         }
-        g_free (escaped_value);
-        g_free (value);
+        escaped_value = g_match_info_fetch (match_info, 2);
+        if (escaped_value && escaped_value[0] != '\0') {
+          value = g_uri_unescape_string (escaped_value, NULL);
+          type_grlkey = GRL_METADATA_KEY_GET_TYPE (grlkey);
+          if (type_grlkey == G_TYPE_STRING) {
+            grl_related_keys_set_string (relkeys, grlkey, value);
+          } else if (type_grlkey == G_TYPE_INT) {
+            grl_related_keys_set_int (relkeys, grlkey, atoi (value));
+          } else if (type_grlkey == G_TYPE_FLOAT) {
+            grl_related_keys_set_float (relkeys, grlkey, atof (value));
+          } else if (type_grlkey == G_TYPE_BOOLEAN) {
+            grl_related_keys_set_boolean (relkeys, grlkey, atoi (value) == 0? FALSE: TRUE);
+          } else if (type_grlkey == G_TYPE_BYTE_ARRAY) {
+            blob = g_base64_decode (value, &blob_size);
+            grl_related_keys_set_binary (relkeys, grlkey, blob, blob_size);
+            g_free (blob);
+          } else if (type_grlkey == G_TYPE_DATE_TIME) {
+            datetime = grl_date_time_from_iso8601 (value);
+            grl_related_keys_set_boxed (relkeys, grlkey, datetime);
+            g_date_time_unref (datetime);
+          }
+          g_free (escaped_value);
+          g_free (value);
+        }
+        if (append) {
+          relkeys_list = g_list_append (relkeys_list, relkeys);
+          g_hash_table_insert (grlkey_related_table,
+                               GRLKEYID_TO_POINTER (grlkey_index),
+                               relkeys_list);
+        }
+        grlkey_count[grlkey]++;
       }
       g_free (keyname);
       g_match_info_next (match_info, NULL);
     }
+    /* Now we can add all the GrlRelatedKeys into media */
+    g_hash_table_foreach (grlkey_related_table,
+                          (GHFunc) _insert_and_free_related_list,
+                          GRL_DATA (media));
+    g_hash_table_unref (grlkey_related_table);
     g_match_info_free (match_info);
     g_free (query);
+    g_free (grlkey_count);
   }
 
   return media;
@@ -690,6 +816,82 @@ grl_media_set_publication_date (GrlMedia *media, const GDateTime *date)
                       date);
 }
 
+/**
+ * grl_media_set_region:
+ * @media: a #GrlMedia
+ * @region: the region's ISO-3166-1 code
+ *
+ * Sets the @region where @media got published.
+ *
+ * Since: 0.2.3
+ */
+void
+grl_media_set_region (GrlMedia *media,
+                      const gchar *region)
+{
+  grl_data_set_string (GRL_DATA (media), GRL_METADATA_KEY_REGION, region);
+}
+
+/**
+ * grl_media_set_region_data:
+ * @media: a #GrlMedia
+ * @region: the region's ISO-3166-1 code
+ * @publication_date: the publication date
+ * @certificate: the age certification
+ *
+ * Sets regional publication and certification information for @region.
+ *
+ * Since: 0.2.3
+ */
+void
+grl_media_set_region_data (GrlMedia *media,
+                           const gchar *region,
+                           const GDateTime *publication_date,
+                           const gchar *certificate)
+{
+  GrlRelatedKeys *relkeys = grl_related_keys_new ();
+  grl_related_keys_set_string (relkeys,
+                               GRL_METADATA_KEY_REGION,
+                               region);
+  grl_related_keys_set_boxed (relkeys,
+                              GRL_METADATA_KEY_PUBLICATION_DATE,
+                              publication_date);
+  grl_related_keys_set_string (relkeys,
+                               GRL_METADATA_KEY_CERTIFICATE,
+                               certificate);
+  grl_data_set_related_keys (GRL_DATA (media), relkeys, 0);
+}
+
+/**
+ * grl_media_add_region_data:
+ * @media: a #GrlMedia
+ * @region: the region's ISO-3166-1 code
+ * @publication_date: the publication date
+ * @certificate: the age certification
+ *
+ * Adds regional publication and certification information for @region.
+ *
+ * Since: 0.2.3
+ */
+void
+grl_media_add_region_data (GrlMedia *media,
+                           const gchar *region,
+                           const GDateTime *publication_date,
+                           const gchar *certificate)
+{
+  GrlRelatedKeys *relkeys = grl_related_keys_new ();
+  grl_related_keys_set_string (relkeys,
+                               GRL_METADATA_KEY_REGION,
+                               region);
+  grl_related_keys_set_boxed (relkeys,
+                              GRL_METADATA_KEY_PUBLICATION_DATE,
+                              publication_date);
+  grl_related_keys_set_string (relkeys,
+                               GRL_METADATA_KEY_CERTIFICATE,
+                               certificate);
+  grl_data_add_related_keys (GRL_DATA (media), relkeys);
+}
+
 /**
   * grl_media_set_creation_date:
   * @media: the media
@@ -849,9 +1051,9 @@ grl_media_set_studio (GrlMedia *media, const gchar *studio)
 /**
  * grl_media_set_certificate:
  * @media: the media
- * @certificate: The rating certificate of the media
+ * @certificate: The age certificate of the media
  *
- * Set the media certificate
+ * Set the media's age certificatication
  *
  * Since: 0.1.6
  */
@@ -880,6 +1082,41 @@ grl_media_set_license (GrlMedia *media, const gchar *license)
                        license);
 }
 
+/**
+ * grl_media_set_favourite:
+ * @media: a media
+ * @favourite: whether the item is favourite or not
+ *
+ * Set if the media is favourite or not
+ *
+ * Since: 0.2.3
+ */
+void
+grl_media_set_favourite (GrlMedia *media, gboolean favourite)
+{
+  grl_data_set_boolean (GRL_DATA (media),
+                        GRL_METADATA_KEY_FAVOURITE,
+                        favourite);
+}
+
+/**
+ * grl_media_set_keyword:
+ * @media: a #GrlMedia
+ * @keyword: a keyword describing the media
+ *
+ * Sets the keyword describing the @media.
+ *
+ * Since: 0.2.3
+ */
+void
+grl_media_set_keyword (GrlMedia *media,
+                       const gchar *keyword)
+{
+  grl_data_set_string (GRL_DATA (media),
+                       GRL_METADATA_KEY_KEYWORD,
+                       keyword);
+}
+
 /**
  * grl_media_get_id:
  * @media: the media object
@@ -1157,6 +1394,80 @@ grl_media_get_publication_date (GrlMedia *media)
                              GRL_METADATA_KEY_PUBLICATION_DATE);
 }
 
+/**
+ * grl_media_get_region:
+ * @media: the media object
+ *
+ * Returns: (transfer none): the ISO-3166-1 of the region where the media got
+ * published (owned by @media).
+ *
+ * Since: 0.2.3
+ */
+const gchar *
+grl_media_get_region (GrlMedia *media)
+{
+  return grl_data_get_string (GRL_DATA (media), GRL_METADATA_KEY_REGION);
+}
+
+/**
+ * grl_media_get_region_data:
+ * @media: the media object
+ * @publication_date: (out) (transfer none): the publication date, or %NULL to ignore.
+ * @certificate: (out) (transfer none): the age certification, or %NULL to ignore.
+ *
+ * Returns: (transfer none): the ISO-3166-1 of the region where the media got
+ * published (owned by @media).
+ *
+ * Since: 0.2.3
+ */
+const gchar *
+grl_media_get_region_data (GrlMedia *media,
+                           const GDateTime **publication_date,
+                           const gchar **certificate)
+{
+  return grl_media_get_region_data_nth (media, 0, publication_date,  certificate);
+}
+
+/**
+ * grl_media_get_region_data_nth:
+ * @media: the media object
+ * @index: element to retrieve
+ * @publication_date: (out) (transfer none): the publication date, or %NULL to ignore.
+ * @certificate: (out) (transfer none): the age certification, or %NULL to ignore.
+ *
+ * Returns: (transfer none): the ISO-3166-1 of the region where the media got
+ * published (owned by @media).
+ *
+ * Since: 0.2.3
+ */
+const gchar *
+grl_media_get_region_data_nth (GrlMedia *media,
+                               guint index,
+                               const GDateTime **publication_date,
+                               const gchar **certificate)
+{
+    GrlRelatedKeys *relkeys =
+      grl_data_get_related_keys (GRL_DATA (media),
+                                 GRL_METADATA_KEY_PUBLICATION_DATE,
+                                 index);
+
+    if (!relkeys) {
+      return NULL;
+    }
+
+    if (publication_date) {
+      *publication_date = grl_related_keys_get_boxed
+                (relkeys, GRL_METADATA_KEY_PUBLICATION_DATE);
+    }
+
+    if (certificate) {
+      *certificate = grl_related_keys_get_string
+                (relkeys, GRL_METADATA_KEY_CERTIFICATE);
+    }
+
+    return grl_related_keys_get_string (relkeys, GRL_METADATA_KEY_REGION);
+}
+
 /**
  * grl_media_get_creation_date:
  * @media: the media
@@ -1355,7 +1666,7 @@ grl_media_get_studio(GrlMedia *media)
  * grl_media_get_certificate:
  * @media: the media object
  *
- * Returns: the media's certificate
+ * Returns: the media's age certificatification
  *
  * Since: 0.1.6
  */
@@ -1393,3 +1704,58 @@ grl_media_get_start_time (GrlMedia *media)
 {
   return grl_data_get_float (GRL_DATA (media), GRL_METADATA_KEY_START_TIME);
 }
+
+/**
+ * grl_media_get_favourite:
+ * @media: the media object
+ *
+ * Returns: whether the media is favourite or not
+ *
+ * Since: 0.2.3
+ */
+gboolean
+grl_media_get_favourite (GrlMedia *media)
+{
+  return grl_data_get_boolean (GRL_DATA (media), GRL_METADATA_KEY_FAVOURITE);
+}
+
+/**
+ * grl_media_get_keyword:
+ * @media: a #GrlMedia
+ *
+ * Returns: (transfer none): the keyword describing the @media (owned by @media).
+ *
+ * Since: 0.2.3
+ */
+const gchar *
+grl_media_get_keyword (GrlMedia *media)
+{
+  return grl_data_get_string (GRL_DATA (media),
+                              GRL_METADATA_KEY_KEYWORD);
+}
+
+/**
+ * grl_media_get_keyword_nth:
+ * @media: a #GrlMedia
+ * @index: element to retrieve
+ *
+ * Returns: (transfer none): the keyword describing the @media (owned by @media).
+ *
+ * Since: 0.2.3
+ */
+const gchar *
+grl_media_get_keyword_nth (GrlMedia *media,
+                           guint index)
+{
+  GrlRelatedKeys *const relkeys =
+    grl_data_get_related_keys (GRL_DATA (media),
+                               GRL_METADATA_KEY_KEYWORD,
+                               index);
+
+  if (!relkeys) {
+    return NULL;
+  }
+
+  return grl_related_keys_get_string (relkeys,
+                                      GRL_METADATA_KEY_KEYWORD);
+}
