@@ -60,6 +60,10 @@ GRL_LOG_DOMAIN_STATIC(test_ui_log_domain);
 #define VIMEO_KEY      "4d908c69e05a9d5b5c6669d302f920cb"
 #define VIMEO_SECRET   "4a923ffaab6238eb"
 
+/* ----- TMDb Config tokens ---- */
+
+#define TMDB_KEY "719b9b296835b04cd919c4bf5220828a"
+
 /* ----- Other ----- */
 
 #define BROWSE_FLAGS (GRL_RESOLVE_FAST_ONLY | GRL_RESOLVE_IDLE_RELAY)
@@ -728,7 +732,7 @@ browse_search_query_cb (GrlSource *source,
 	      grl_source_search (source,
 				       state->text,
 				       all_keys (),
-				       options,
+				       supported_options,
 				       browse_search_query_cb,
 				       state);
 	    break;
@@ -737,7 +741,7 @@ browse_search_query_cb (GrlSource *source,
 	      grl_source_query (source,
 				      state->text,
 				      all_keys (),
-				      options,
+				      supported_options,
 				      browse_search_query_cb,
 				      state);
 	    break;
@@ -844,19 +848,12 @@ static void
 resolve (GrlSource *source, GrlMedia *media)
 {
   if (source) {
-    /* If source does not support resolve() operation, then use the current
-       media */
-    if ((grl_source_supported_operations (source) &
-         GRL_OP_RESOLVE)) {
-      grl_source_resolve (source,
-                          media,
-                          all_keys (),
-                          default_resolve_options,
-                          resolve_cb,
-                          NULL);
-    } else {
-      resolve_cb (source, 0, media, NULL, NULL);
-    }
+    grl_source_resolve (source,
+                        media,
+                        all_keys (),
+                        default_resolve_options,
+                        resolve_cb,
+                        NULL);
   }
 }
 
@@ -1181,6 +1178,7 @@ search (GrlSource *source, const gchar *text)
 					 supported_options,
 					 browse_search_query_cb,
 					 state);
+    g_object_unref (supported_options);
   } else {
     /* Multiple search (all sources) */
     multiple = TRUE;
@@ -1192,7 +1190,6 @@ search (GrlSource *source, const gchar *text)
 				     browse_search_query_cb,
 				     state);
   }
-  g_object_unref (supported_options);
   clear_panes ();
   operation_started (source, search_id, multiple);
 }
@@ -1642,6 +1639,19 @@ set_vimeo_config (void)
   grl_registry_add_config (registry, config, NULL);
 }
 
+static void
+set_tmdb_config (void)
+{
+  GrlConfig *config;
+  GrlRegistry *registry;
+
+  config =grl_config_new ("grl-tmdb", NULL);
+  grl_config_set_api_key (config, TMDB_KEY);
+
+  registry = grl_registry_get_default ();
+  grl_registry_add_config (registry, config, NULL);
+}
+
 static void
 set_local_config (void)
 {
@@ -2203,6 +2213,7 @@ configure_plugins ()
   set_flickr_config ();
   set_youtube_config ();
   set_vimeo_config ();
+  set_tmdb_config ();
   set_local_config ();
 }
 
